@@ -24,8 +24,6 @@
  * SECTION:gst
  * @short_description: Media library supporting arbitrary formats and filter
  *                     graphs.
- * @see_also: Check out both <ulink url="http://www.cse.ogi.edu/sysl/">OGI's
- *            pipeline</ulink> and Microsoft's DirectShow for some background.
  *
  * GStreamer is a framework for constructing graphs of various filters
  * (termed elements here) that will handle streaming media.  Any discreet
@@ -34,11 +32,8 @@
  * a powerful negotiation framework.  Plugins are heavily used to provide for
  * all elements, allowing one to construct plugins outside of the GST
  * library, even released binary-only if license require (please don't).
- *
- * GStreamer borrows heavily from both the <ulink
- * url="http://www.cse.ogi.edu/sysl/">OGI media pipeline</ulink> and
- * Microsoft's DirectShow, hopefully taking the best of both and leaving the
- * cruft behind. Its interface is slowly getting stable.
+ * GStreamer covers a wide range of use cases including: playback, recording,
+ * editing, serving streams, voice over ip and video calls.
  *
  * The <application>GStreamer</application> library should be initialized with
  * gst_init() before it can be used. You should pass pointers to the main argc
@@ -108,9 +103,6 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <sys/types.h>
-#ifdef HAVE_FORK
-#include <sys/wait.h>
-#endif /* HAVE_FORK */
 #ifdef HAVE_SYS_UTSNAME_H
 #include <sys/utsname.h>
 #endif
@@ -135,8 +127,14 @@
 static gboolean gst_initialized = FALSE;
 static gboolean gst_deinitialized = FALSE;
 
+#ifdef G_OS_WIN32
+HMODULE _priv_gst_dll_handle = NULL;
+#endif
+
 #ifndef GST_DISABLE_REGISTRY
-static GList *plugin_paths = NULL;      /* for delayed processing in post_init */
+GList *_priv_gst_plugin_paths = NULL;   /* for delayed processing in post_init */
+
+extern gboolean _priv_gst_disable_registry_update;
 #endif
 
 #ifndef GST_DISABLE_GST_DEBUG
@@ -144,22 +142,10 @@ extern const gchar *priv_gst_dump_dot_dir;
 #endif
 
 /* defaults */
-#if defined(HAVE_FORK) && !defined(GST_HAVE_UNSAFE_FORK)
-#define DEFAULT_FORK TRUE
-#else
-#define DEFAULT_FORK FALSE
-#endif /* HAVE_FORK */
 
 /* set to TRUE when segfaults need to be left as is */
 static gboolean _gst_disable_segtrap = FALSE;
 
-/* control the behaviour of registry rebuild */
-static gboolean _gst_enable_registry_fork = DEFAULT_FORK;
-
-/*set to TRUE when registry needn't to be updated */
-static gboolean _gst_disable_registry_update = FALSE;
-
-static void load_plugin_func (gpointer data, gpointer user_data);
 static gboolean init_pre (GOptionContext * context, GOptionGroup * group,
     gpointer data, GError ** error);
 static gboolean init_post (GOptionContext * context, GOptionGroup * group,
@@ -169,7 +155,7 @@ static gboolean parse_goption_arg (const gchar * s_opt,
     const gchar * arg, gpointer data, GError ** err);
 #endif
 
-static GSList *preload_plugins = NULL;
+GSList *_priv_gst_preload_plugins = NULL;
 
 const gchar g_log_domain_gstreamer[] = "GStreamer";
 
@@ -254,7 +240,7 @@ parse_debug_list (const gchar * list)
   gchar **split;
   gchar **walk;
 
-  g_return_if_fail (list != NULL);
+  g_assert (list);
 
   split = g_strsplit (list, ",", 0);
 
@@ -284,6 +270,18 @@ parse_debug_list (const gchar * list)
 }
 #endif
 
+#ifdef G_OS_WIN32
+BOOL WINAPI DllMain (HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved);
+BOOL WINAPI
+DllMain (HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved)
+{
+  if (fdwReason == DLL_PROCESS_ATTACH)
+    _priv_gst_dll_handle = (HMODULE) hinstDLL;
+  return TRUE;
+}
+
+#endif
+
 /**
  * gst_init_get_option_group:
  *
@@ -299,7 +297,7 @@ parse_debug_list (const gchar * list)
  * threading system as one of the very first things in your program
  * (see the example at the beginning of this section).
  *
- * Returns: a pointer to GStreamer's option group.
+ * Returns: (transfer full): a pointer to GStreamer's option group.
  */
 
 GOptionGroup *
@@ -357,25 +355,31 @@ gst_init_get_option_group (void)
     {"gst-disable-registry-fork", 0, G_OPTION_FLAG_NO_ARG,
           G_OPTION_ARG_CALLBACK,
           (gpointer) parse_goption_arg,
-          N_("Disable the use of fork() while scanning the registry"),
+          N_("Disable spawning a helper process while scanning the registry"),
         NULL},
     {NULL}
   };
 
-  /* The GLib threading system must be initialised before calling any other
-   * GLib function according to the documentation; if the application hasn't
-   * called gst_init() yet or initialised the threading system otherwise, we
-   * better issue a warning here (since chances are high that the application
-   * has already called other GLib functions such as g_option_context_new() */
-  if (!g_thread_supported ()) {
-    g_warning ("The GStreamer function gst_init_get_option_group() was\n"
-        "\tcalled, but the GLib threading system has not been initialised\n"
-        "\tyet, something that must happen before any other GLib function\n"
-        "\tis called. The application needs to be fixed so that it calls\n"
-        "\t   if (!g_thread_supported ()) g_thread_init(NULL);\n"
-        "\tas very first thing in its main() function. Please file a bug\n"
-        "\tagainst this application.");
-    g_thread_init (NULL);
+  /* Since GLib 2.23.2 calling g_thread_init() 'late' is allowed and is
+   * automatically done as part of g_type_init() */
+  if (glib_check_version (2, 23, 3)) {
+    /* The GLib threading system must be initialised before calling any other
+     * GLib function according to the documentation; if the application hasn't
+     * called gst_init() yet or initialised the threading system otherwise, we
+     * better issue a warning here (since chances are high that the application
+     * has already called other GLib functions such as g_option_context_new() */
+    if (!g_thread_get_initialized ()) {
+      g_warning ("The GStreamer function gst_init_get_option_group() was\n"
+          "\tcalled, but the GLib threading system has not been initialised\n"
+          "\tyet, something that must happen before any other GLib function\n"
+          "\tis called. The application needs to be fixed so that it calls\n"
+          "\t   if (!g_thread_get_initialized ()) g_thread_init(NULL);\n"
+          "\tas very first thing in its main() function. Please file a bug\n"
+          "\tagainst this application.");
+      g_thread_init (NULL);
+    }
+  } else {
+    /* GLib >= 2.23.2 */
   }
 
   group = g_option_group_new ("gst", _("GStreamer Options"),
@@ -394,8 +398,8 @@ gst_init_get_option_group (void)
 
 /**
  * gst_init_check:
- * @argc: pointer to application's argc
- * @argv: pointer to application's argv
+ * @argc: (inout) (allow-none): pointer to application's argc
+ * @argv: (inout) (array length=argc) (allow-none): pointer to application's argv
  * @err: pointer to a #GError to which a message will be posted on error
  *
  * Initializes the GStreamer library, setting up internal path lists,
@@ -420,7 +424,7 @@ gst_init_check (int *argc, char **argv[], GError ** err)
 #endif
   gboolean res;
 
-  if (!g_thread_supported ())
+  if (!g_thread_get_initialized ())
     g_thread_init (NULL);
 
   if (gst_initialized) {
@@ -453,12 +457,19 @@ gst_init_check (int *argc, char **argv[], GError ** err)
 
 /**
  * gst_init:
- * @argc: pointer to application's argc
- * @argv: pointer to application's argv
+ * @argc: (inout) (allow-none): pointer to application's argc
+ * @argv: (inout) (array length=argc) (allow-none): pointer to application's argv
  *
  * Initializes the GStreamer library, setting up internal path lists,
  * registering built-in elements, and loading standard plugins.
  *
+ * Unless the plugin registry is disabled at compile time, the registry will be
+ * loaded. By default this will also check if the registry cache needs to be
+ * updated and rescan all plugins if needed. See gst_update_registry() for
+ * details and section
+ * <link linkend="gst-running">Running GStreamer Applications</link>
+ * for how to disable automatic registry updates.
+ *
  * This function should be called before calling any other GLib functions. If
  * this is not an option, your program must initialise the GLib thread system
  * using g_thread_init() before any other GLib functions are called.
@@ -489,12 +500,29 @@ gst_init (int *argc, char **argv[])
   }
 }
 
+/**
+ * gst_is_initialized:
+ *
+ * Use this function to check if GStreamer has been initialized with gst_init()
+ * or gst_init_check().
+ *
+ * Returns: TRUE if initialization has been done, FALSE otherwise.
+ *
+ * Since: 0.10.31
+ */
+gboolean
+gst_is_initialized (void)
+{
+  return gst_initialized;
+}
+
 #ifndef GST_DISABLE_REGISTRY
 static void
 add_path_func (gpointer data, gpointer user_data)
 {
   GST_INFO ("Adding plugin path: \"%s\", will scan later", (gchar *) data);
-  plugin_paths = g_list_append (plugin_paths, g_strdup (data));
+  _priv_gst_plugin_paths =
+      g_list_append (_priv_gst_plugin_paths, g_strdup (data));
 }
 #endif
 
@@ -502,40 +530,15 @@ add_path_func (gpointer data, gpointer user_data)
 static void
 prepare_for_load_plugin_func (gpointer data, gpointer user_data)
 {
-  preload_plugins = g_slist_prepend (preload_plugins, g_strdup (data));
+  _priv_gst_preload_plugins =
+      g_slist_prepend (_priv_gst_preload_plugins, g_strdup (data));
 }
 #endif
 
-static void
-load_plugin_func (gpointer data, gpointer user_data)
-{
-  GstPlugin *plugin;
-  const gchar *filename;
-  GError *err = NULL;
-
-  filename = (const gchar *) data;
-
-  plugin = gst_plugin_load_file (filename, &err);
-
-  if (plugin) {
-    GST_INFO ("Loaded plugin: \"%s\"", filename);
-
-    gst_default_registry_add_plugin (plugin);
-  } else {
-    if (err) {
-      /* Report error to user, and free error */
-      GST_ERROR ("Failed to load plugin: %s", err->message);
-      g_error_free (err);
-    } else {
-      GST_WARNING ("Failed to load plugin: \"%s\"", filename);
-    }
-  }
-}
-
 #ifndef GST_DISABLE_OPTION_PARSING
 static void
-split_and_iterate (const gchar * stringlist, gchar * separator, GFunc iterator,
-    gpointer user_data)
+split_and_iterate (const gchar * stringlist, const gchar * separator,
+    GFunc iterator, gpointer user_data)
 {
   gchar **strings;
   gint j = 0;
@@ -569,20 +572,11 @@ init_pre (GOptionContext * context, GOptionGroup * group, gpointer data,
     return TRUE;
   }
 
-  /* GStreamer was built against a GLib >= 2.8 and is therefore not doing
-   * the refcount hack. Check that it isn't being run against an older GLib */
-  if (glib_major_version < 2 ||
-      (glib_major_version == 2 && glib_minor_version < 8)) {
-    g_warning ("GStreamer was compiled against GLib %d.%d.%d but is running"
-        " against %d.%d.%d. This will cause reference counting issues",
-        GLIB_MAJOR_VERSION, GLIB_MINOR_VERSION, GLIB_MICRO_VERSION,
-        glib_major_version, glib_minor_version, glib_micro_version);
-  }
-
   g_type_init ();
 
   /* we need threading to be enabled right here */
-  g_assert (g_thread_supported ());
+  g_assert (g_thread_get_initialized ());
+
   _gst_debug_init ();
 
 #ifdef ENABLE_NLS
@@ -641,338 +635,6 @@ gst_register_core_elements (GstPlugin * plugin)
   return TRUE;
 }
 
-#ifndef GST_DISABLE_REGISTRY
-
-typedef enum
-{
-  REGISTRY_SCAN_AND_UPDATE_FAILURE = 0,
-  REGISTRY_SCAN_AND_UPDATE_SUCCESS_NOT_CHANGED,
-  REGISTRY_SCAN_AND_UPDATE_SUCCESS_UPDATED
-} GstRegistryScanAndUpdateResult;
-
-/*
- * scan_and_update_registry:
- * @default_registry: the #GstRegistry
- * @registry_file: registry filename
- * @write_changes: write registry if it has changed?
- *
- * Scans for registry changes and eventually updates the registry cache. 
- *
- * Return: %REGISTRY_SCAN_AND_UPDATE_FAILURE if the registry could not scanned
- *         or updated, %REGISTRY_SCAN_AND_UPDATE_SUCCESS_NOT_CHANGED if the
- *         registry is clean and %REGISTRY_SCAN_AND_UPDATE_SUCCESS_UPDATED if
- *         it has been updated and the cache needs to be re-read.
- */
-static GstRegistryScanAndUpdateResult
-scan_and_update_registry (GstRegistry * default_registry,
-    const gchar * registry_file, gboolean write_changes, GError ** error)
-{
-  const gchar *plugin_path;
-  gboolean changed = FALSE;
-  GList *l;
-
-  GST_INFO ("Validating registry cache: %s", registry_file);
-  /* It sounds tempting to just compare the mtime of directories with the mtime
-   * of the registry cache, but it does not work. It would not catch updated
-   * plugins, which might bring more or less features.
-   */
-
-  /* scan paths specified via --gst-plugin-path */
-  GST_DEBUG ("scanning paths added via --gst-plugin-path");
-  for (l = plugin_paths; l != NULL; l = l->next) {
-    GST_INFO ("Scanning plugin path: \"%s\"", (gchar *) l->data);
-    changed |= gst_registry_scan_path (default_registry, (gchar *) l->data);
-  }
-  /* keep plugin_paths around in case a re-scan is forced later on */
-
-  /* GST_PLUGIN_PATH specifies a list of directories to scan for
-   * additional plugins.  These take precedence over the system plugins */
-  plugin_path = g_getenv ("GST_PLUGIN_PATH");
-  if (plugin_path) {
-    char **list;
-    int i;
-
-    GST_DEBUG ("GST_PLUGIN_PATH set to %s", plugin_path);
-    list = g_strsplit (plugin_path, G_SEARCHPATH_SEPARATOR_S, 0);
-    for (i = 0; list[i]; i++) {
-      changed |= gst_registry_scan_path (default_registry, list[i]);
-    }
-    g_strfreev (list);
-  } else {
-    GST_DEBUG ("GST_PLUGIN_PATH not set");
-  }
-
-  /* GST_PLUGIN_SYSTEM_PATH specifies a list of plugins that are always
-   * loaded by default.  If not set, this defaults to the system-installed
-   * path, and the plugins installed in the user's home directory */
-  plugin_path = g_getenv ("GST_PLUGIN_SYSTEM_PATH");
-  if (plugin_path == NULL) {
-    char *home_plugins;
-
-    GST_DEBUG ("GST_PLUGIN_SYSTEM_PATH not set");
-
-    /* plugins in the user's home directory take precedence over
-     * system-installed ones */
-    home_plugins = g_build_filename (g_get_home_dir (),
-        ".gstreamer-" GST_MAJORMINOR, "plugins", NULL);
-    GST_DEBUG ("scanning home plugins %s", home_plugins);
-    changed |= gst_registry_scan_path (default_registry, home_plugins);
-    g_free (home_plugins);
-
-    /* add the main (installed) library path */
-    GST_DEBUG ("scanning main plugins %s", PLUGINDIR);
-    changed |= gst_registry_scan_path (default_registry, PLUGINDIR);
-
-#ifdef G_OS_WIN32
-    {
-      char *base_dir;
-      char *dir;
-
-      base_dir = g_win32_get_package_installation_directory (NULL,
-          "libgstreamer-0.10-0.dll");
-
-      dir = g_build_filename (base_dir, "lib", "gstreamer-0.10", NULL);
-      GST_DEBUG ("scanning DLL dir %s", dir);
-
-      changed |= gst_registry_scan_path (default_registry, dir);
-
-      g_free (dir);
-      g_free (base_dir);
-    }
-#endif
-  } else {
-    gchar **list;
-    gint i;
-
-    GST_DEBUG ("GST_PLUGIN_SYSTEM_PATH set to %s", plugin_path);
-    list = g_strsplit (plugin_path, G_SEARCHPATH_SEPARATOR_S, 0);
-    for (i = 0; list[i]; i++) {
-      changed |= gst_registry_scan_path (default_registry, list[i]);
-    }
-    g_strfreev (list);
-  }
-
-  /* Remove cached plugins so stale info is cleared. */
-  changed |= _priv_gst_registry_remove_cache_plugins (default_registry);
-
-  if (!changed) {
-    GST_INFO ("Registry cache has not changed");
-    return REGISTRY_SCAN_AND_UPDATE_SUCCESS_NOT_CHANGED;
-  }
-
-  if (!write_changes) {
-    GST_INFO ("Registry cache changed, but writing is disabled. Not writing.");
-    return REGISTRY_SCAN_AND_UPDATE_FAILURE;
-  }
-
-  GST_INFO ("Registry cache changed. Writing new registry cache");
-#ifdef USE_BINARY_REGISTRY
-  if (!gst_registry_binary_write_cache (default_registry, registry_file)) {
-#else
-  if (!gst_registry_xml_write_cache (default_registry, registry_file)) {
-#endif
-    g_set_error (error, GST_CORE_ERROR, GST_CORE_ERROR_FAILED,
-        _("Error writing registry cache to %s: %s"),
-        registry_file, g_strerror (errno));
-    return REGISTRY_SCAN_AND_UPDATE_FAILURE;
-  }
-
-  GST_INFO ("Registry cache written successfully");
-  return REGISTRY_SCAN_AND_UPDATE_SUCCESS_UPDATED;
-}
-
-static gboolean
-ensure_current_registry_nonforking (GstRegistry * default_registry,
-    const gchar * registry_file, GError ** error)
-{
-  /* fork() not available */
-  GST_DEBUG ("Updating registry cache in-process");
-  scan_and_update_registry (default_registry, registry_file, TRUE, error);
-  return TRUE;
-}
-
-/* when forking is not available this function always does nothing but return
- * TRUE immediatly */
-static gboolean
-ensure_current_registry_forking (GstRegistry * default_registry,
-    const gchar * registry_file, GError ** error)
-{
-#ifdef HAVE_FORK
-  pid_t pid;
-  int pfd[2];
-  int ret;
-
-  /* We fork here, and let the child read and possibly rebuild the registry.
-   * After that, the parent will re-read the freshly generated registry. */
-  GST_DEBUG ("forking to update registry");
-
-  if (pipe (pfd) == -1) {
-    g_set_error (error, GST_CORE_ERROR, GST_CORE_ERROR_FAILED,
-        _("Error re-scanning registry %s: %s"),
-        ", could not create pipes. Error", g_strerror (errno));
-    return FALSE;
-  }
-
-  pid = fork ();
-  if (pid == -1) {
-    GST_ERROR ("Failed to fork()");
-    g_set_error (error, GST_CORE_ERROR, GST_CORE_ERROR_FAILED,
-        _("Error re-scanning registry %s: %s"),
-        ", failed to fork. Error", g_strerror (errno));
-    return FALSE;
-  }
-
-  if (pid == 0) {
-    gint result_code;
-
-    /* this is the child. Close the read pipe */
-    (void) close (pfd[0]);
-
-    GST_DEBUG ("child reading registry cache");
-    result_code =
-        scan_and_update_registry (default_registry, registry_file, TRUE, NULL);
-
-    /* need to use _exit, so that any exit handlers registered don't
-     * bring down the main program */
-    GST_DEBUG ("child exiting: %d", result_code);
-
-    /* make valgrind happy (yes, you can call it insane) */
-    g_free ((char *) registry_file);
-
-    /* write a result byte to the pipe */
-    do {
-      ret = write (pfd[1], &result_code, sizeof (result_code));
-    } while (ret == -1 && errno == EINTR);
-    /* if ret == -1 now, we could not write to pipe, probably 
-     * means parent has exited before us */
-    (void) close (pfd[1]);
-
-    _exit (0);
-  } else {
-    gint result_code;
-
-    /* parent. Close write pipe */
-    (void) close (pfd[1]);
-
-    /* Wait for result from the pipe */
-    GST_DEBUG ("Waiting for data from child");
-    do {
-      ret = read (pfd[0], &result_code, sizeof (result_code));
-    } while (ret == -1 && errno == EINTR);
-
-    if (ret == -1) {
-      g_set_error (error, GST_CORE_ERROR, GST_CORE_ERROR_FAILED,
-          _("Error re-scanning registry %s: %s"),
-          ", read returned error", g_strerror (errno));
-      close (pfd[0]);
-      return FALSE;
-    }
-    (void) close (pfd[0]);
-
-    /* Wait to ensure the child is reaped, but ignore the result */
-    GST_DEBUG ("parent waiting on child");
-    waitpid (pid, NULL, 0);
-    GST_DEBUG ("parent done waiting on child");
-
-    if (ret == 0) {
-      GST_ERROR ("child did not exit normally, terminated by signal");
-      g_set_error (error, GST_CORE_ERROR, GST_CORE_ERROR_FAILED,
-          _("Error re-scanning registry %s"), ", child terminated by signal");
-      return FALSE;
-    }
-
-    if (result_code == REGISTRY_SCAN_AND_UPDATE_SUCCESS_UPDATED) {
-      GST_DEBUG ("Child succeeded. Parent reading registry cache");
-      _priv_gst_registry_remove_cache_plugins (default_registry);
-#ifdef USE_BINARY_REGISTRY
-      gst_registry_binary_read_cache (default_registry, registry_file);
-#else
-      gst_registry_xml_read_cache (default_registry, registry_file);
-#endif
-    } else if (result_code == REGISTRY_SCAN_AND_UPDATE_FAILURE) {
-      GST_DEBUG ("Child failed. Parent re-scanning registry, ignoring errors.");
-      scan_and_update_registry (default_registry, registry_file, FALSE, NULL);
-    }
-  }
-#endif /* HAVE_FORK */
-  return TRUE;
-}
-
-static gboolean
-ensure_current_registry (GError ** error)
-{
-  gchar *registry_file;
-  GstRegistry *default_registry;
-  gboolean ret = TRUE;
-  gboolean do_fork;
-  gboolean do_update;
-  gboolean have_cache;
-
-  default_registry = gst_registry_get_default ();
-  registry_file = g_strdup (g_getenv ("GST_REGISTRY"));
-  if (registry_file == NULL) {
-#ifdef USE_BINARY_REGISTRY
-    registry_file = g_build_filename (g_get_home_dir (),
-        ".gstreamer-" GST_MAJORMINOR, "registry." HOST_CPU ".bin", NULL);
-#else
-    registry_file = g_build_filename (g_get_home_dir (),
-        ".gstreamer-" GST_MAJORMINOR, "registry." HOST_CPU ".xml", NULL);
-#endif
-  }
-
-  GST_INFO ("reading registry cache: %s", registry_file);
-#ifdef USE_BINARY_REGISTRY
-  have_cache = gst_registry_binary_read_cache (default_registry, registry_file);
-#else
-  have_cache = gst_registry_xml_read_cache (default_registry, registry_file);
-#endif
-
-  if (have_cache) {
-    do_update = !_gst_disable_registry_update;
-    if (do_update) {
-      const gchar *update_env;
-
-      if ((update_env = g_getenv ("GST_REGISTRY_UPDATE"))) {
-        /* do update for any value different from "no" */
-        do_update = (strcmp (update_env, "no") != 0);
-      }
-    }
-  } else {
-    do_update = TRUE;
-  }
-
-  if (do_update) {
-    /* first see if forking is enabled */
-    do_fork = _gst_enable_registry_fork;
-    if (do_fork) {
-      const gchar *fork_env;
-
-      /* forking enabled, see if it is disabled with an env var */
-      if ((fork_env = g_getenv ("GST_REGISTRY_FORK"))) {
-        /* fork enabled for any value different from "no" */
-        do_fork = strcmp (fork_env, "no") != 0;
-      }
-    }
-
-    /* now check registry with or without forking */
-    if (do_fork) {
-      GST_DEBUG ("forking for registry rebuild");
-      ret = ensure_current_registry_forking (default_registry, registry_file,
-          error);
-    } else {
-      GST_DEBUG ("requested not to fork for registry rebuild");
-      ret = ensure_current_registry_nonforking (default_registry, registry_file,
-          error);
-    }
-  }
-
-  g_free (registry_file);
-  GST_INFO ("registry reading and updating done, result = %d", ret);
-
-  return ret;
-}
-#endif /* GST_DISABLE_REGISTRY */
-
 /*
  * this bit handles:
  * - initalization of threads if we use them
@@ -1022,12 +684,14 @@ init_post (GOptionContext * context, GOptionGroup * group, gpointer data,
   g_type_class_ref (gst_bin_flags_get_type ());
   g_type_class_ref (gst_buffer_flag_get_type ());
   g_type_class_ref (gst_buffer_copy_flags_get_type ());
+  g_type_class_ref (gst_buffer_list_item_get_type ());
   g_type_class_ref (gst_bus_flags_get_type ());
   g_type_class_ref (gst_bus_sync_reply_get_type ());
   g_type_class_ref (gst_caps_flags_get_type ());
   g_type_class_ref (gst_clock_return_get_type ());
   g_type_class_ref (gst_clock_entry_type_get_type ());
   g_type_class_ref (gst_clock_flags_get_type ());
+  g_type_class_ref (gst_clock_type_get_type ());
   g_type_class_ref (gst_debug_graph_details_get_type ());
   g_type_class_ref (gst_state_get_type ());
   g_type_class_ref (gst_state_change_return_get_type ());
@@ -1055,6 +719,7 @@ init_post (GOptionContext * context, GOptionGroup * group, gpointer data,
   g_type_class_ref (gst_message_type_get_type ());
   g_type_class_ref (gst_mini_object_flags_get_type ());
   g_type_class_ref (gst_pad_link_return_get_type ());
+  g_type_class_ref (gst_pad_link_check_get_type ());
   g_type_class_ref (gst_flow_return_get_type ());
   g_type_class_ref (gst_activate_mode_get_type ());
   g_type_class_ref (gst_pad_direction_get_type ());
@@ -1068,8 +733,11 @@ init_post (GOptionContext * context, GOptionGroup * group, gpointer data,
   g_type_class_ref (gst_rank_get_type ());
   g_type_class_ref (gst_query_type_get_type ());
   g_type_class_ref (gst_buffering_mode_get_type ());
+  g_type_class_ref (gst_stream_status_type_get_type ());
+  g_type_class_ref (gst_structure_change_type_get_type ());
   g_type_class_ref (gst_tag_merge_mode_get_type ());
   g_type_class_ref (gst_tag_flag_get_type ());
+  g_type_class_ref (gst_task_pool_get_type ());
   g_type_class_ref (gst_task_state_get_type ());
   g_type_class_ref (gst_alloc_trace_flags_get_type ());
   g_type_class_ref (gst_type_find_probability_get_type ());
@@ -1084,6 +752,7 @@ init_post (GOptionContext * context, GOptionGroup * group, gpointer data,
   gst_caps_get_type ();
   _gst_event_initialize ();
   _gst_buffer_initialize ();
+  _gst_buffer_list_initialize ();
   _gst_message_initialize ();
   _gst_tag_initialize ();
 
@@ -1108,14 +777,8 @@ init_post (GOptionContext * context, GOptionGroup * group, gpointer data,
    */
   gst_initialized = TRUE;
 
-#ifndef GST_DISABLE_REGISTRY
-  if (!ensure_current_registry (error))
+  if (!gst_update_registry ())
     return FALSE;
-#endif /* GST_DISABLE_REGISTRY */
-
-  /* if we need to preload plugins, do so now */
-  g_slist_foreach (preload_plugins, load_plugin_func, NULL);
-  /* keep preload_plugins around in case a re-scan is forced later on */
 
 #ifndef GST_DISABLE_TRACE
   _gst_trace_on = 0;
@@ -1125,6 +788,11 @@ init_post (GOptionContext * context, GOptionGroup * group, gpointer data,
   }
 #endif /* GST_DISABLE_TRACE */
 
+  GST_INFO ("GLib runtime version: %d.%d.%d", glib_major_version,
+      glib_minor_version, glib_micro_version);
+  GST_INFO ("GLib headers version: %d.%d.%d", GLIB_MAJOR_VERSION,
+      GLIB_MINOR_VERSION, GLIB_MICRO_VERSION);
+
   return TRUE;
 }
 
@@ -1264,10 +932,12 @@ parse_one_option (gint opt, const gchar * arg, GError ** err)
       _gst_disable_segtrap = TRUE;
       break;
     case ARG_REGISTRY_UPDATE_DISABLE:
-      _gst_disable_registry_update = TRUE;
+#ifndef GST_DISABLE_REGISTRY
+      _priv_gst_disable_registry_update = TRUE;
+#endif
       break;
     case ARG_REGISTRY_FORK_DISABLE:
-      _gst_enable_registry_fork = FALSE;
+      gst_registry_fork_set_enabled (FALSE);
       break;
     default:
       g_set_error (err, G_OPTION_ERROR, G_OPTION_ERROR_UNKNOWN_OPTION,
@@ -1284,7 +954,7 @@ parse_goption_arg (const gchar * opt,
 {
   static const struct
   {
-    gchar *opt;
+    const gchar *opt;
     int val;
   } options[] = {
     {
@@ -1320,8 +990,6 @@ parse_goption_arg (const gchar * opt,
 }
 #endif
 
-extern GstRegistry *_gst_registry_default;
-
 /**
  * gst_deinit:
  *
@@ -1346,14 +1014,14 @@ gst_deinit (void)
     return;
   }
 
-  g_slist_foreach (preload_plugins, (GFunc) g_free, NULL);
-  g_slist_free (preload_plugins);
-  preload_plugins = NULL;
+  g_slist_foreach (_priv_gst_preload_plugins, (GFunc) g_free, NULL);
+  g_slist_free (_priv_gst_preload_plugins);
+  _priv_gst_preload_plugins = NULL;
 
 #ifndef GST_DISABLE_REGISTRY
-  g_list_foreach (plugin_paths, (GFunc) g_free, NULL);
-  g_list_free (plugin_paths);
-  plugin_paths = NULL;
+  g_list_foreach (_priv_gst_plugin_paths, (GFunc) g_free, NULL);
+  g_list_free (_priv_gst_plugin_paths);
+  _priv_gst_plugin_paths = NULL;
 #endif
 
   clock = gst_system_clock_obtain ();
@@ -1375,9 +1043,11 @@ gst_deinit (void)
   g_type_class_unref (g_type_class_peek (gst_bin_flags_get_type ()));
   g_type_class_unref (g_type_class_peek (gst_buffer_flag_get_type ()));
   g_type_class_unref (g_type_class_peek (gst_buffer_copy_flags_get_type ()));
+  g_type_class_unref (g_type_class_peek (gst_buffer_list_item_get_type ()));
   g_type_class_unref (g_type_class_peek (gst_bus_flags_get_type ()));
   g_type_class_unref (g_type_class_peek (gst_bus_sync_reply_get_type ()));
   g_type_class_unref (g_type_class_peek (gst_caps_flags_get_type ()));
+  g_type_class_unref (g_type_class_peek (gst_clock_type_get_type ()));
   g_type_class_unref (g_type_class_peek (gst_clock_return_get_type ()));
   g_type_class_unref (g_type_class_peek (gst_clock_entry_type_get_type ()));
   g_type_class_unref (g_type_class_peek (gst_clock_flags_get_type ()));
@@ -1388,8 +1058,15 @@ gst_deinit (void)
   g_type_class_unref (g_type_class_peek (gst_element_flags_get_type ()));
   g_type_class_unref (g_type_class_peek (gst_core_error_get_type ()));
   g_type_class_unref (g_type_class_peek (gst_library_error_get_type ()));
+  g_type_class_unref (g_type_class_peek (gst_plugin_dependency_flags_get_type
+          ()));
+  g_type_class_unref (g_type_class_peek (gst_parse_flags_get_type ()));
   g_type_class_unref (g_type_class_peek (gst_resource_error_get_type ()));
+  g_type_class_unref (g_type_class_peek (gst_search_mode_get_type ()));
   g_type_class_unref (g_type_class_peek (gst_stream_error_get_type ()));
+  g_type_class_unref (g_type_class_peek (gst_stream_status_type_get_type ()));
+  g_type_class_unref (g_type_class_peek (gst_structure_change_type_get_type
+          ()));
   g_type_class_unref (g_type_class_peek (gst_event_type_flags_get_type ()));
   g_type_class_unref (g_type_class_peek (gst_event_type_get_type ()));
   g_type_class_unref (g_type_class_peek (gst_seek_type_get_type ()));
@@ -1409,6 +1086,7 @@ gst_deinit (void)
   g_type_class_unref (g_type_class_peek (gst_message_type_get_type ()));
   g_type_class_unref (g_type_class_peek (gst_mini_object_flags_get_type ()));
   g_type_class_unref (g_type_class_peek (gst_pad_link_return_get_type ()));
+  g_type_class_unref (g_type_class_peek (gst_pad_link_check_get_type ()));
   g_type_class_unref (g_type_class_peek (gst_flow_return_get_type ()));
   g_type_class_unref (g_type_class_peek (gst_activate_mode_get_type ()));
   g_type_class_unref (g_type_class_peek (gst_pad_direction_get_type ()));
@@ -1437,10 +1115,10 @@ gst_deinit (void)
 
 /**
  * gst_version:
- * @major: pointer to a guint to store the major version number
- * @minor: pointer to a guint to store the minor version number
- * @micro: pointer to a guint to store the micro version number
- * @nano:  pointer to a guint to store the nano version number
+ * @major: (out): pointer to a guint to store the major version number
+ * @minor: (out): pointer to a guint to store the minor version number
+ * @micro: (out): pointer to a guint to store the micro version number
+ * @nano:  (out): pointer to a guint to store the nano version number
  *
  * Gets the version number of the GStreamer library.
  */
@@ -1464,11 +1142,12 @@ gst_version (guint * major, guint * minor, guint * micro, guint * nano)
  * This function returns a string that is useful for describing this version
  * of GStreamer to the outside world: user agent strings, logging, ...
  *
- * Returns: a newly allocated string describing this version of GStreamer.
+ * Returns: (transfer full): a newly allocated string describing this version
+ *     of GStreamer.
  */
 
 gchar *
-gst_version_string ()
+gst_version_string (void)
 {
   guint major, minor, micro, nano;
 
@@ -1518,95 +1197,3 @@ gst_segtrap_set_enabled (gboolean enabled)
 {
   _gst_disable_segtrap = !enabled;
 }
-
-/**
- * gst_registry_fork_is_enabled:
- *
- * By default GStreamer will perform a fork() when scanning and rebuilding the
- * registry file. 
- *
- * Applications might want to disable this behaviour with the
- * gst_registry_fork_set_enabled() function. 
- *
- * Returns: %TRUE if GStreamer will use fork() when rebuilding the registry. On
- * platforms without fork(), this function will always return %FALSE.
- *
- * Since: 0.10.10
- */
-gboolean
-gst_registry_fork_is_enabled (void)
-{
-  return _gst_enable_registry_fork;
-}
-
-/**
- * gst_registry_fork_set_enabled:
- * @enabled: whether rebuilding the registry may fork
- *
- * Applications might want to disable/enable the usage of fork() when rebuilding
- * the registry. See gst_registry_fork_is_enabled() for more information.
- *
- * On platforms without fork(), this function will have no effect on the return
- * value of gst_registry_fork_is_enabled().
- *
- * Since: 0.10.10
- */
-void
-gst_registry_fork_set_enabled (gboolean enabled)
-{
-#ifdef HAVE_FORK
-  _gst_enable_registry_fork = enabled;
-#endif /* HAVE_FORK */
-}
-
-
-/**
- * gst_update_registry:
- *
- * Forces GStreamer to re-scan its plugin paths and update the default
- * plugin registry.
- *
- * Applications will almost never need to call this function, it is only
- * useful if the application knows new plugins have been installed (or old
- * ones removed) since the start of the application (or, to be precise, the
- * first call to gst_init()) and the application wants to make use of any
- * newly-installed plugins without restarting the application.
- *
- * Applications should assume that the registry update is neither atomic nor
- * thread-safe and should therefore not have any dynamic pipelines running
- * (including the playbin and decodebin elements) and should also not create
- * any elements or access the GStreamer registry while the update is in
- * progress.
- *
- * Note that this function may block for a significant amount of time.
- *
- * Returns: %TRUE if the registry has been updated successfully (does not
- *          imply that there were changes), otherwise %FALSE.
- *
- * Since: 0.10.12
- */
-gboolean
-gst_update_registry (void)
-{
-  gboolean res = FALSE;
-
-#ifndef GST_DISABLE_REGISTRY
-  GError *err = NULL;
-
-  res = ensure_current_registry (&err);
-  if (err) {
-    GST_WARNING ("registry update failed: %s", err->message);
-    g_error_free (err);
-  } else {
-    GST_LOG ("registry update succeeded");
-  }
-
-  if (preload_plugins) {
-    g_slist_foreach (preload_plugins, load_plugin_func, NULL);
-  }
-#else
-  GST_WARNING ("registry update failed: %s", "registry disabled");
-#endif /* GST_DISABLE_REGISTRY */
-
-  return res;
-}
