@@ -235,6 +235,7 @@ G_STMT_START {						\
 typedef struct _GstClockEntry	GstClockEntry;
 typedef struct _GstClock	GstClock;
 typedef struct _GstClockClass	GstClockClass;
+typedef struct _GstClockPrivate	GstClockPrivate;
 
 /* --- prototype for async callbacks --- */
 /**
@@ -259,6 +260,7 @@ typedef gboolean	(*GstClockCallback)	(GstClock *clock, GstClockTime time,
  * @GST_CLOCK_BADTIME: A bad time was provided to a function.
  * @GST_CLOCK_ERROR: An error occurred
  * @GST_CLOCK_UNSUPPORTED: Operation is not supported
+ * @GST_CLOCK_DONE: The ClockID is done waiting
  *
  * The return value of a clock operation.
  */
@@ -270,7 +272,8 @@ typedef enum
   GST_CLOCK_BUSY	=  3,
   GST_CLOCK_BADTIME	=  4,
   GST_CLOCK_ERROR	=  5,
-  GST_CLOCK_UNSUPPORTED	=  6
+  GST_CLOCK_UNSUPPORTED	=  6,
+  GST_CLOCK_DONE	=  7
 } GstClockReturn;
 
 /**
@@ -334,6 +337,8 @@ typedef enum {
  *
  * All pending timeouts or periodic notifies are converted into
  * an entry.
+ * Note that GstClockEntry should be treated as an opaque structure. It must
+ * not be extended or allocated using a custom allocator.
  */
 struct _GstClockEntry {
   gint			refcount;
@@ -345,6 +350,9 @@ struct _GstClockEntry {
   GstClockReturn	 status;
   GstClockCallback	 func;
   gpointer		 user_data;
+  GDestroyNotify	 destroy_data;
+  gboolean               unscheduled;
+  gboolean               woken_up;
 };
 
 /**
@@ -447,7 +455,10 @@ struct _GstClock {
   GstClockID     clockid;
 
   /*< private >*/
-  GstClockTime	 _gst_reserved[GST_PADDING];
+  union {
+    GstClockPrivate *priv;
+    GstClockTime     _gst_reserved[GST_PADDING];
+  } ABI;
 };
 
 /**
@@ -540,8 +551,15 @@ GstClockReturn		gst_clock_id_wait		(GstClockID id,
 GstClockReturn		gst_clock_id_wait_async		(GstClockID id,
 							 GstClockCallback func,
 							 gpointer user_data);
+GstClockReturn		gst_clock_id_wait_async_full	(GstClockID id,
+							 GstClockCallback func,
+							 gpointer user_data,
+							 GDestroyNotify destroy_data);
 void			gst_clock_id_unschedule		(GstClockID id);
 
+gboolean                gst_clock_single_shot_id_reinit (GstClock * clock,
+							 GstClockID id,
+							 GstClockTime time);
 
 G_END_DECLS
 
