@@ -279,8 +279,8 @@ gst_event_new (GstEventType type)
 /**
  * gst_event_new_custom:
  * @type: The type of the new event
- * @structure: The structure for the event. The event will take ownership of
- * the structure.
+ * @structure: (transfer full): the structure for the event. The event will
+ *     take ownership of the structure.
  *
  * Create a new custom-typed event. This can be used for anything not
  * handled by other event-specific functions to pass an event to another
@@ -293,7 +293,7 @@ gst_event_new (GstEventType type)
  * New custom events can also be created by subclassing the event type if
  * needed.
  *
- * Returns: The new custom event.
+ * Returns: (transfer full): the new custom event.
  */
 GstEvent *
 gst_event_new_custom (GstEventType type, GstStructure * structure)
@@ -431,7 +431,7 @@ gst_event_set_seqnum (GstEvent * event, guint32 seqnum)
  * This event is typically generated after a seek to flush out all queued data
  * in the pipeline so that the new media is played as soon as possible.
  *
- * Returns: A new flush start event.
+ * Returns: (transfer full): a new flush start event.
  */
 GstEvent *
 gst_event_new_flush_start (void)
@@ -453,7 +453,7 @@ gst_event_new_flush_start (void)
  * This event is typically generated to complete a seek and to resume
  * dataflow.
  *
- * Returns: A new flush stop event.
+ * Returns: (transfer full): a new flush stop event.
  */
 GstEvent *
 gst_event_new_flush_stop (void)
@@ -478,7 +478,7 @@ gst_event_new_flush_stop (void)
  *
  * The EOS event itself will not cause any state transitions of the pipeline.
  *
- * Returns: The new EOS event.
+ * Returns: (transfer full): the new EOS event.
  */
 GstEvent *
 gst_event_new_eos (void)
@@ -500,7 +500,7 @@ gst_event_new_eos (void)
  * This method calls gst_event_new_new_segment_full() passing a default
  * value of 1.0 for applied_rate
  *
- * Returns: A new newsegment event.
+ * Returns: (transfer full): a new newsegment event.
  */
 GstEvent *
 gst_event_new_new_segment (gboolean update, gdouble rate, GstFormat format,
@@ -513,12 +513,12 @@ gst_event_new_new_segment (gboolean update, gdouble rate, GstFormat format,
 /**
  * gst_event_parse_new_segment:
  * @event: The event to query
- * @update: A pointer to the update flag of the segment
- * @rate: A pointer to the rate of the segment
- * @format: A pointer to the format of the newsegment values
- * @start: A pointer to store the start value in
- * @stop: A pointer to store the stop value in
- * @position: A pointer to store the stream time in
+ * @update: (out): A pointer to the update flag of the segment
+ * @rate: (out): A pointer to the rate of the segment
+ * @format: (out): A pointer to the format of the newsegment values
+ * @start: (out): A pointer to store the start value in
+ * @stop: (out): A pointer to store the stop value in
+ * @position: (out): A pointer to store the stream time in
  *
  * Get the update flag, rate, format, start, stop and position in the 
  * newsegment event. In general, gst_event_parse_new_segment_full() should
@@ -575,7 +575,7 @@ gst_event_parse_new_segment (GstEvent * event, gboolean * update,
  *
  *   position + (TIMESTAMP(buf) - start) * ABS (rate * applied_rate)
  *
- * Returns: A new newsegment event.
+ * Returns: (transfer full): a new newsegment event.
  *
  * Since: 0.10.6
  */
@@ -626,13 +626,13 @@ gst_event_new_new_segment_full (gboolean update, gdouble rate,
 /**
  * gst_event_parse_new_segment_full:
  * @event: The event to query
- * @update: A pointer to the update flag of the segment
- * @rate: A pointer to the rate of the segment
- * @applied_rate: A pointer to the applied_rate of the segment
- * @format: A pointer to the format of the newsegment values
- * @start: A pointer to store the start value in
- * @stop: A pointer to store the stop value in
- * @position: A pointer to store the stream time in
+ * @update: (out): A pointer to the update flag of the segment
+ * @rate: (out): A pointer to the rate of the segment
+ * @applied_rate: (out): A pointer to the applied_rate of the segment
+ * @format: (out): A pointer to the format of the newsegment values
+ * @start: (out): A pointer to store the start value in
+ * @stop: (out): A pointer to store the stop value in
+ * @position: (out): A pointer to store the stream time in
  *
  * Get the update, rate, applied_rate, format, start, stop and 
  * position in the newsegment event. See gst_event_new_new_segment_full() 
@@ -650,7 +650,7 @@ gst_event_parse_new_segment_full (GstEvent * event, gboolean * update,
   g_return_if_fail (GST_IS_EVENT (event));
   g_return_if_fail (GST_EVENT_TYPE (event) == GST_EVENT_NEWSEGMENT);
 
-  structure = gst_event_get_structure (event);
+  structure = event->structure;
   if (G_LIKELY (update))
     *update =
         g_value_get_boolean (gst_structure_id_get_value (structure,
@@ -687,7 +687,7 @@ gst_event_parse_new_segment_full (GstEvent * event, gboolean * update,
  *
  * Generates a metadata tag event from the given @taglist.
  *
- * Returns: a new #GstEvent
+ * Returns: (transfer full): a new #GstEvent
  */
 GstEvent *
 gst_event_new_tag (GstTagList * taglist)
@@ -700,7 +700,7 @@ gst_event_new_tag (GstTagList * taglist)
 /**
  * gst_event_parse_tag:
  * @event: a tag event
- * @taglist: pointer to metadata list
+ * @taglist: (out) (transfer none): pointer to metadata list
  *
  * Parses a tag @event and stores the results in the given @taglist location.
  */
@@ -727,7 +727,7 @@ gst_event_parse_tag (GstEvent * event, GstTagList ** taglist)
  *
  * When the @async flag is set, a thread boundary is prefered.
  *
- * Returns: a new #GstEvent
+ * Returns: (transfer full): a new #GstEvent
  */
 GstEvent *
 gst_event_new_buffer_size (GstFormat format, gint64 minsize,
@@ -754,10 +754,10 @@ gst_event_new_buffer_size (GstFormat format, gint64 minsize,
 /**
  * gst_event_parse_buffer_size:
  * @event: The event to query
- * @format: A pointer to store the format in
- * @minsize: A pointer to store the minsize in
- * @maxsize: A pointer to store the maxsize in
- * @async: A pointer to store the async-flag in
+ * @format: (out): A pointer to store the format in
+ * @minsize: (out): A pointer to store the minsize in
+ * @maxsize: (out): A pointer to store the maxsize in
+ * @async: (out): A pointer to store the async-flag in
  *
  * Get the format, minsize, maxsize and async-flag in the buffersize event.
  */
@@ -770,7 +770,7 @@ gst_event_parse_buffer_size (GstEvent * event, GstFormat * format,
   g_return_if_fail (GST_IS_EVENT (event));
   g_return_if_fail (GST_EVENT_TYPE (event) == GST_EVENT_BUFFERSIZE);
 
-  structure = gst_event_get_structure (event);
+  structure = event->structure;
   if (format)
     *format =
         g_value_get_enum (gst_structure_id_get_value (structure,
@@ -829,7 +829,7 @@ gst_event_parse_buffer_size (GstEvent * event, GstFormat * format,
  * The application can use general event probes to intercept the QoS
  * event and implement custom application specific QoS handling.
  *
- * Returns: A new QOS event.
+ * Returns: (transfer full): a new QOS event.
  */
 GstEvent *
 gst_event_new_qos (gdouble proportion, GstClockTimeDiff diff,
@@ -858,9 +858,9 @@ gst_event_new_qos (gdouble proportion, GstClockTimeDiff diff,
 /**
  * gst_event_parse_qos:
  * @event: The event to query
- * @proportion: A pointer to store the proportion in
- * @diff: A pointer to store the diff in
- * @timestamp: A pointer to store the timestamp in
+ * @proportion: (out): A pointer to store the proportion in
+ * @diff: (out): A pointer to store the diff in
+ * @timestamp: (out): A pointer to store the timestamp in
  *
  * Get the proportion, diff and timestamp in the qos event. See
  * gst_event_new_qos() for more information about the different QoS values.
@@ -874,7 +874,7 @@ gst_event_parse_qos (GstEvent * event, gdouble * proportion,
   g_return_if_fail (GST_IS_EVENT (event));
   g_return_if_fail (GST_EVENT_TYPE (event) == GST_EVENT_QOS);
 
-  structure = gst_event_get_structure (event);
+  structure = event->structure;
   if (proportion)
     *proportion =
         g_value_get_double (gst_structure_id_get_value (structure,
@@ -931,7 +931,7 @@ gst_event_parse_qos (GstEvent * event, gdouble * proportion,
  * #GST_QUERY_POSITION and update the playback segment current position with a
  * #GST_SEEK_TYPE_SET to the desired position. 
  *
- * Returns: A new seek event.
+ * Returns: (transfer full): a new seek event.
  */
 GstEvent *
 gst_event_new_seek (gdouble rate, GstFormat format, GstSeekFlags flags,
@@ -974,13 +974,13 @@ gst_event_new_seek (gdouble rate, GstFormat format, GstSeekFlags flags,
 /**
  * gst_event_parse_seek:
  * @event: a seek event
- * @rate: result location for the rate
- * @format: result location for the stream format
- * @flags:  result location for the #GstSeekFlags
- * @start_type: result location for the #GstSeekType of the start position
- * @start: result location for the start postion expressed in @format
- * @stop_type:  result location for the #GstSeekType of the stop position
- * @stop: result location for the stop postion expressed in @format
+ * @rate: (out): result location for the rate
+ * @format: (out): result location for the stream format
+ * @flags:  (out): result location for the #GstSeekFlags
+ * @start_type: (out): result location for the #GstSeekType of the start position
+ * @start: (out): result location for the start postion expressed in @format
+ * @stop_type:  (out): result location for the #GstSeekType of the stop position
+ * @stop: (out): result location for the stop postion expressed in @format
  *
  * Parses a seek @event and stores the results in the given result locations.
  */
@@ -994,7 +994,7 @@ gst_event_parse_seek (GstEvent * event, gdouble * rate,
   g_return_if_fail (GST_IS_EVENT (event));
   g_return_if_fail (GST_EVENT_TYPE (event) == GST_EVENT_SEEK);
 
-  structure = gst_event_get_structure (event);
+  structure = event->structure;
   if (rate)
     *rate =
         g_value_get_double (gst_structure_id_get_value (structure,
@@ -1027,12 +1027,12 @@ gst_event_parse_seek (GstEvent * event, gdouble * rate,
 
 /**
  * gst_event_new_navigation:
- * @structure: description of the event. The event will take ownership of the
- *     structure.
+ * @structure: (transfer full): description of the event. The event will take
+ *     ownership of the structure.
  *
  * Create a new navigation event from the given description.
  *
- * Returns: a new #GstEvent
+ * Returns: (transfer full): a new #GstEvent
  */
 GstEvent *
 gst_event_new_navigation (GstStructure * structure)
@@ -1053,7 +1053,7 @@ gst_event_new_navigation (GstStructure * structure)
  * The latency is mostly used in live sinks and is always expressed in
  * the time format.
  *
- * Returns: a new #GstEvent
+ * Returns: (transfer full): a new #GstEvent
  *
  * Since: 0.10.12
  */
@@ -1076,7 +1076,7 @@ gst_event_new_latency (GstClockTime latency)
 /**
  * gst_event_parse_latency:
  * @event: The event to query
- * @latency: A pointer to store the latency in.
+ * @latency: (out): A pointer to store the latency in.
  *
  * Get the latency in the latency event.
  *
@@ -1085,15 +1085,12 @@ gst_event_new_latency (GstClockTime latency)
 void
 gst_event_parse_latency (GstEvent * event, GstClockTime * latency)
 {
-  const GstStructure *structure;
-
   g_return_if_fail (GST_IS_EVENT (event));
   g_return_if_fail (GST_EVENT_TYPE (event) == GST_EVENT_LATENCY);
 
-  structure = gst_event_get_structure (event);
   if (latency)
     *latency =
-        g_value_get_uint64 (gst_structure_id_get_value (structure,
+        g_value_get_uint64 (gst_structure_id_get_value (event->structure,
             GST_QUARK (LATENCY)));
 }
 
@@ -1118,7 +1115,7 @@ gst_event_parse_latency (GstEvent * event, GstClockTime * latency)
  * The @intermediate flag instructs the pipeline that this step operation is
  * part of a larger step operation.
  *
- * Returns: a new #GstEvent
+ * Returns: (transfer full): a new #GstEvent
  *
  * Since: 0.10.24
  */
@@ -1147,11 +1144,12 @@ gst_event_new_step (GstFormat format, guint64 amount, gdouble rate,
 /**
  * gst_event_parse_step:
  * @event: The event to query
- * @format: A pointer to store the format in.
- * @amount: A pointer to store the amount in.
- * @rate: A pointer to store the rate in.
- * @flush: A pointer to store the flush boolean in.
- * @intermediate: A pointer to store the intermediate boolean in.
+ * @format: (out) (allow-none): a pointer to store the format in
+ * @amount: (out) (allow-none): a pointer to store the amount in
+ * @rate: (out) (allow-none): a pointer to store the rate in
+ * @flush: (out) (allow-none): a pointer to store the flush boolean in
+ * @intermediate: (out) (allow-none): a pointer to store the intermediate
+ *     boolean in
  *
  * Parse the step event.
  *
@@ -1166,7 +1164,7 @@ gst_event_parse_step (GstEvent * event, GstFormat * format, guint64 * amount,
   g_return_if_fail (GST_IS_EVENT (event));
   g_return_if_fail (GST_EVENT_TYPE (event) == GST_EVENT_STEP);
 
-  structure = gst_event_get_structure (event);
+  structure = event->structure;
   if (format)
     *format = g_value_get_enum (gst_structure_id_get_value (structure,
             GST_QUARK (FORMAT)));
@@ -1186,13 +1184,13 @@ gst_event_parse_step (GstEvent * event, GstFormat * format, guint64 * amount,
 
 /**
  * gst_event_new_sink_message:
- * @msg: The #GstMessage to be posted
+ * @msg: (transfer none): the #GstMessage to be posted
  *
  * Create a new sink-message event. The purpose of the sink-message event is
  * to instruct a sink to post the message contained in the event synchronized
  * with the stream.
  *
- * Returns: a new #GstEvent
+ * Returns: (transfer full): a new #GstEvent
  *
  * Since: 0.10.26
  */
@@ -1216,7 +1214,7 @@ gst_event_new_sink_message (GstMessage * msg)
 /**
  * gst_event_parse_sink_message:
  * @event: The event to query
- * @msg: A pointer to store the #GstMessage in.
+ * @msg: (out) (transfer full): a pointer to store the #GstMessage in.
  *
  * Parse the sink-message event. Unref @msg after usage.
  *
@@ -1225,14 +1223,11 @@ gst_event_new_sink_message (GstMessage * msg)
 void
 gst_event_parse_sink_message (GstEvent * event, GstMessage ** msg)
 {
-  const GstStructure *structure;
-
   g_return_if_fail (GST_IS_EVENT (event));
   g_return_if_fail (GST_EVENT_TYPE (event) == GST_EVENT_SINK_MESSAGE);
 
-  structure = gst_event_get_structure (event);
   if (msg)
     *msg =
         GST_MESSAGE (gst_value_dup_mini_object (gst_structure_id_get_value
-            (structure, GST_QUARK (MESSAGE))));
+            (event->structure, GST_QUARK (MESSAGE))));
 }
