@@ -29,6 +29,8 @@
  * in a pipeline.
  */
 
+/* FIXME: complete gobject annotations */
+
 #include "gst_private.h"
 
 #include "gstinfo.h"
@@ -58,8 +60,6 @@ enum
 GST_DEBUG_CATEGORY_STATIC (index_debug);
 #define GST_CAT_DEFAULT index_debug
 
-static void gst_index_class_init (GstIndexClass * klass);
-static void gst_index_init (GstIndex * index);
 static void gst_index_finalize (GObject * object);
 
 static void gst_index_set_property (GObject * object, guint prop_id,
@@ -68,6 +68,7 @@ static void gst_index_get_property (GObject * object, guint prop_id,
     GValue * value, GParamSpec * pspec);
 
 static GstIndexGroup *gst_index_group_new (guint groupnum);
+static void gst_index_group_free (GstIndexGroup * group);
 
 static gboolean gst_index_path_resolver (GstIndex * index, GstObject * writer,
     gchar ** writer_string, gpointer data);
@@ -151,9 +152,9 @@ gst_index_class_init (GstIndexClass * klass)
       G_STRUCT_OFFSET (GstIndexClass, entry_added), NULL, NULL,
       gst_marshal_VOID__BOXED, G_TYPE_NONE, 1, GST_TYPE_INDEX_ENTRY);
 
-  gobject_class->set_property = GST_DEBUG_FUNCPTR (gst_index_set_property);
-  gobject_class->get_property = GST_DEBUG_FUNCPTR (gst_index_get_property);
-  gobject_class->finalize = GST_DEBUG_FUNCPTR (gst_index_finalize);
+  gobject_class->set_property = gst_index_set_property;
+  gobject_class->get_property = gst_index_get_property;
+  gobject_class->finalize = gst_index_finalize;
 
   g_object_class_install_property (gobject_class, ARG_RESOLVER,
       g_param_spec_enum ("resolver", "Resolver",
@@ -198,7 +199,7 @@ gst_index_finalize (GObject * object)
   GstIndex *index = GST_INDEX (object);
 
   if (index->groups) {
-    g_list_foreach (index->groups, (GFunc) g_free, NULL);
+    g_list_foreach (index->groups, (GFunc) gst_index_group_free, NULL);
     g_list_free (index->groups);
     index->groups = NULL;
   }
@@ -259,7 +260,7 @@ gst_index_get_property (GObject * object, guint prop_id,
 static GstIndexGroup *
 gst_index_group_new (guint groupnum)
 {
-  GstIndexGroup *indexgroup = g_new (GstIndexGroup, 1);
+  GstIndexGroup *indexgroup = g_slice_new (GstIndexGroup);
 
   indexgroup->groupnum = groupnum;
   indexgroup->entries = NULL;
@@ -271,19 +272,25 @@ gst_index_group_new (guint groupnum)
   return indexgroup;
 }
 
+static void
+gst_index_group_free (GstIndexGroup * group)
+{
+  g_slice_free (GstIndexGroup, group);
+}
+
 /**
  * gst_index_new:
  *
  * Create a new tileindex object
  *
- * Returns: a new index object
+ * Returns: (transfer full): a new index object
  */
 GstIndex *
 gst_index_new (void)
 {
   GstIndex *index;
 
-  index = g_object_new (gst_index_get_type (), NULL);
+  index = g_object_newv (gst_index_get_type (), 0, NULL);
 
   return index;
 }
@@ -496,17 +503,22 @@ gst_index_set_resolver_full (GstIndex * index, GstIndexResolver resolver,
  *
  * Copies an entry and returns the result.
  *
- * Returns: a newly allocated #GstIndexEntry.
+ * Free-function: gst_index_entry_free
+ *
+ * Returns: (transfer full): a newly allocated #GstIndexEntry.
  */
 GstIndexEntry *
 gst_index_entry_copy (GstIndexEntry * entry)
 {
-  return g_memdup (entry, sizeof (*entry));
+  GstIndexEntry *new_entry = g_slice_new (GstIndexEntry);
+
+  memcpy (new_entry, entry, sizeof (GstIndexEntry));
+  return new_entry;
 }
 
 /**
  * gst_index_entry_free:
- * @entry: the entry to free
+ * @entry: (transfer full): the entry to free
  *
  * Free the memory used by the given entry.
  */
@@ -532,7 +544,7 @@ gst_index_entry_free (GstIndexEntry * entry)
       break;
   }
 
-  g_free (entry);
+  g_slice_free (GstIndexEntry, entry);
 }
 
 /**
@@ -545,7 +557,9 @@ gst_index_entry_free (GstIndexEntry * entry)
  * used to map dynamic GstFormat ids to their original
  * format key.
  *
- * Returns: a pointer to the newly added entry in the index.
+ * Free-function: gst_index_entry_free
+ *
+ * Returns: (transfer full): a pointer to the newly added entry in the index.
  */
 GstIndexEntry *
 gst_index_add_format (GstIndex * index, gint id, GstFormat format)
@@ -559,7 +573,7 @@ gst_index_add_format (GstIndex * index, gint id, GstFormat format)
   if (!GST_INDEX_IS_WRITABLE (index) || id == -1)
     return NULL;
 
-  entry = g_new0 (GstIndexEntry, 1);
+  entry = g_slice_new (GstIndexEntry);
   entry->type = GST_INDEX_ENTRY_FORMAT;
   entry->id = id;
   entry->data.format.format = format;
@@ -593,7 +607,7 @@ gst_index_add_id (GstIndex * index, gint id, gchar * description)
   if (!GST_INDEX_IS_WRITABLE (index) || id == -1)
     return NULL;
 
-  entry = g_new0 (GstIndexEntry, 1);
+  entry = g_slice_new (GstIndexEntry);
   entry->type = GST_INDEX_ENTRY_ID;
   entry->id = id;
   entry->data.id.description = description;
@@ -616,6 +630,8 @@ static gboolean
 gst_index_gtype_resolver (GstIndex * index, GstObject * writer,
     gchar ** writer_string, gpointer data)
 {
+  g_return_val_if_fail (writer != NULL, FALSE);
+
   if (GST_IS_PAD (writer)) {
     GstElement *element =
         (GstElement *) gst_object_get_parent (GST_OBJECT (writer));
@@ -699,7 +715,7 @@ gst_index_get_writer_id (GstIndex * index, GstObject * writer, gint * id)
     if (!entry) {
       /* index is probably not writable, make an entry anyway
        * to keep it in our cache */
-      entry = g_new0 (GstIndexEntry, 1);
+      entry = g_slice_new (GstIndexEntry);
       entry->type = GST_INDEX_ENTRY_ID;
       entry->id = *id;
       entry->data.id.description = writer_string;
@@ -723,7 +739,7 @@ gst_index_add_entry (GstIndex * index, GstIndexEntry * entry)
     iclass->add_entry (index, entry);
   }
 
-  g_signal_emit (G_OBJECT (index), gst_index_signals[ENTRY_ADDED], 0, entry);
+  g_signal_emit (index, gst_index_signals[ENTRY_ADDED], 0, entry);
 }
 
 /**
@@ -751,7 +767,7 @@ gst_index_add_associationv (GstIndex * index, gint id, GstAssocFlags flags,
   if (!GST_INDEX_IS_WRITABLE (index) || id == -1)
     return NULL;
 
-  entry = g_malloc (sizeof (GstIndexEntry));
+  entry = g_slice_new (GstIndexEntry);
 
   entry->type = GST_INDEX_ENTRY_ASSOCIATION;
   entry->id = id;
