@@ -344,10 +344,12 @@ preset_get_keyfile (GstPreset * preset)
 
     /* compare version to check for merge */
     if (in_system) {
-      if (!in_user || preset_parse_version (str_version_system) >
+      /* keep system presets if there is no user preset or when the system
+       * version is higher than the user version. */
+      if (!in_user) {
+        presets = in_system;
+      } else if (preset_parse_version (str_version_system) >
           preset_parse_version (str_version_user)) {
-        /* keep system presets if there is no user preset or when the system
-         * version is higher than the user version. */
         presets = in_system;
         updated_from_system = TRUE;
       }
@@ -865,7 +867,8 @@ no_presets:
  *
  * Get a copy of preset names as a NULL terminated string array.
  *
- * Returns: list with names, ue g_strfreev() after usage.
+ * Returns: (transfer full) (array zero-terminated=1) (element-type gchar*):
+ *     list with names, ue g_strfreev() after usage.
  *
  * Since: 0.10.20
  */
@@ -883,7 +886,8 @@ gst_preset_get_preset_names (GstPreset * preset)
  *
  * Get a the names of the GObject properties that can be used for presets.
  *
- * Returns: an array of property names which should be freed with g_strfreev() after use.
+ * Returns: (transfer full) (array zero-terminated=1) (element-type gchar*): an
+ *   array of property names which should be freed with g_strfreev() after use.
  *
  * Since: 0.10.20
  */
@@ -920,8 +924,8 @@ gst_preset_load_preset (GstPreset * preset, const gchar * name)
  * @preset: a #GObject that implements #GstPreset
  * @name: preset name to save
  *
- * Save the current preset under the given name. If there is already a preset by
- * this @name it will be overwritten.
+ * Save the current object settings as a preset under the given name. If there
+ * is already a preset by this @name it will be overwritten.
  *
  * Returns: %TRUE for success, %FALSE
  *
@@ -1012,7 +1016,7 @@ gst_preset_set_meta (GstPreset * preset, const gchar * name, const gchar * tag,
  * @preset: a #GObject that implements #GstPreset
  * @name: preset name
  * @tag: meta data item name
- * @value: value
+ * @value: (out callee-allocates): value
  *
  * Gets the @value for an existing meta data @tag. Meta data @tag names can be
  * something like e.g. "comment". Returned values need to be released when done.
