@@ -24,7 +24,6 @@
 #ifndef __GST_REGISTRY_H__
 #define __GST_REGISTRY_H__
 
-#include <stdio.h> /* FIXME: because of cache_file below */
 #include <gst/gstconfig.h>
 #include <gst/gstplugin.h>
 #include <gst/gstpluginfeature.h>
@@ -40,6 +39,7 @@ G_BEGIN_DECLS
 
 typedef struct _GstRegistry GstRegistry;
 typedef struct _GstRegistryClass GstRegistryClass;
+typedef struct _GstRegistryPrivate GstRegistryPrivate;
 
 /**
  * GstRegistry:
@@ -60,9 +60,13 @@ struct _GstRegistry {
 
   /* hash to speedup _lookup_feature_locked() */
   GHashTable *feature_hash;
+  /* hash to speedup _lookup */
+  GHashTable *basename_hash;
+
+  GstRegistryPrivate *priv;
 
   /*< private >*/
-  gpointer _gst_reserved[GST_PADDING-1];
+  gpointer _gst_reserved[GST_PADDING-3];
 };
 
 struct _GstRegistryClass {
@@ -103,6 +107,7 @@ GList*			gst_registry_feature_filter	(GstRegistry *registry,
 GList *                 gst_registry_get_feature_list   (GstRegistry *registry,
                                                          GType type);
 GList *                 gst_registry_get_feature_list_by_plugin (GstRegistry *registry, const gchar *name);
+guint32                 gst_registry_get_feature_list_cookie (GstRegistry *registry);
 
 GstPlugin*		gst_registry_find_plugin	(GstRegistry *registry, const gchar *name);
 GstPluginFeature*	gst_registry_find_feature	(GstRegistry *registry, const gchar *name, GType type);
@@ -110,15 +115,19 @@ GstPluginFeature*	gst_registry_find_feature	(GstRegistry *registry, const gchar
 GstPlugin * 		gst_registry_lookup 		(GstRegistry *registry, const char *filename);
 GstPluginFeature * 	gst_registry_lookup_feature 	(GstRegistry *registry, const char *name);
 
-/* FIXME 0.11: do we really want to export these? (If yes, we should add a GError argument) */
+/* These are only here because at some point they were in a public header
+ * (even though they should have been private) and we can't really remove
+ * them now (FIXME: 0.11). They don't do anything other than return FALSE. */
+#ifndef GST_DISABLE_DEPRECATED
 gboolean 		gst_registry_xml_read_cache 	(GstRegistry * registry, const char *location);
 gboolean 		gst_registry_xml_write_cache 	(GstRegistry * registry, const char *location);
+#endif
 
 /* convinience defines for the default registry */
 
 /**
  * gst_default_registry_add_plugin:
- * @plugin: the plugin to add
+ * @plugin: (transfer full): the plugin to add
  *
  * Add the plugin to the default registry.
  * The plugin-added signal will be emitted.
@@ -144,7 +153,8 @@ gboolean 		gst_registry_xml_write_cache 	(GstRegistry * registry, const char *lo
  *
  * Get the list of paths for the default registry.
  *
- * Returns: A Glist of paths as strings. g_list_free after use.
+ * Returns: (transfer container) (element-type char*): a #GList of paths as
+ *     strings. g_list_free() after use.
  */
 #define gst_default_registry_get_path_list() \
   gst_registry_get_path_list (gst_registry_get_default())
@@ -154,7 +164,8 @@ gboolean 		gst_registry_xml_write_cache 	(GstRegistry * registry, const char *lo
  *
  * Get a copy of all plugins registered in the default registry.
  *
- * Returns: a copy of the list. Free after use.
+ * Returns: (transfer full) (element-type Gst.Plugin): a copy of the list.
+ *     Free after use.
  */
 #define gst_default_registry_get_plugin_list() \
   gst_registry_get_plugin_list (gst_registry_get_default())
@@ -166,8 +177,8 @@ gboolean 		gst_registry_xml_write_cache 	(GstRegistry * registry, const char *lo
  *
  * Find the pluginfeature with the given name and type in the default registry.
  *
- * Returns: The pluginfeature with the given name and type or NULL
- * if the plugin was not found.
+ * Returns: (transfer full): the pluginfeature with the given name and type or
+ *    NULL if the plugin was not found.
  */
 #define gst_default_registry_find_feature(name,type) \
   gst_registry_find_feature (gst_registry_get_default(),name,type)
@@ -179,7 +190,8 @@ gboolean 		gst_registry_xml_write_cache 	(GstRegistry * registry, const char *lo
  * Find the plugin with the given name in the default registry.
  * The plugin will be reffed; caller is responsible for unreffing.
  *
- * Returns: The plugin with the given name or NULL if the plugin was not found.
+ * Returns: (transfer full): The plugin with the given name or NULL if the
+ *     plugin was not found.
  */
 #define gst_default_registry_find_plugin(name) \
   gst_registry_find_plugin (gst_registry_get_default(),name)
@@ -195,11 +207,25 @@ gboolean 		gst_registry_xml_write_cache 	(GstRegistry * registry, const char *lo
  * If the first flag is set, only the first match is
  * returned (as a list with a single object).
  *
- * Returns: a GList of plugin features, gst_plugin_feature_list_free after use.
+ * Returns: (transfer full) (element-type Gst.PluginFeature): a #GList of
+ *     plugin features, gst_plugin_feature_list_free after use.
  */
 #define gst_default_registry_feature_filter(filter,first,user_data) \
   gst_registry_feature_filter (gst_registry_get_default(),filter,first,user_data)
 
+/**
+ * gst_default_registry_get_feature_list_cookie:
+ *
+ * Returns the default registrys feature list cookie. This changes
+ * every time a feature is added or removed from the registry.
+ *
+ * Returns: the feature list cookie.
+ *
+ * Since: 0.10.26
+ */
+#define gst_default_registry_get_feature_list_cookie() \
+  gst_registry_get_feature_list_cookie (gst_registry_get_default())
+
 gboolean                gst_default_registry_check_feature_version (const gchar *feature_name,
                                                                     guint        min_major,
                                                                     guint        min_minor,
